/*
  Copyright (c) 2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "Direction.hpp"

#include <functional>


namespace RPGEdit {

namespace Model {

template<typename T>
struct Position {
	T x, y;

	Position translate(Direction dir, T amount) const {
		Position p = *this;

		switch (dir) {
		case Direction::NORTH:
			p.y -= amount;
			break;

		case Direction::EAST:
			p.x += amount;
			break;

		case Direction::SOUTH:
			p.y += amount;
			break;

		case Direction::WEST:
			p.x -= amount;
		}

		return p;
	}

	Position<T> operator+(Direction dir) const {
		return translate(dir, 1);
	}

	bool operator==(const Position<T> &p) const {
		return (x == p.x) && (y == p.y);
	}

	template<typename T2>
	explicit operator Position<T2>() const {
		return Position<T2>{T2(x), T2(y)};
	}
};

}

}

namespace std {
template<typename T> struct hash<RPGEdit::Model::Position<T>> {
	typedef size_t result_type;
	typedef RPGEdit::Model::Position<T> argument_type;

	size_t operator()(const RPGEdit::Model::Position<T> &pos) const noexcept {
		const int shift = 4*sizeof(size_t); // half the bit count of size_t

		std::hash<T> hash;
		size_t hash1 = hash(pos.x), hash2 = hash(pos.y);

		return hash1 ^ ((hash2 << shift) | (hash2 >> shift));
	}
};

}
